<?php

require_once "PlateauQuantik.php";
class ActionQuantik
{

    protected  $plateau ; //le plateau de jeux

    //crée un plateau
    public function __construct(PlateauQuantik $plateau) {
        $this->plateau = $plateau;
    }

    //retourne un plateau
    public function getPlateau():PlateauQuantik{
        return $this->plateau;
    }

    //Permet de tester si les pieces de la ligne  numRow  forment une combinaison gagnante
    public function isRowWin(int $numRow):bool {

        $pieces = $this->plateau->getRow($numRow);

        return self::isComboWin($pieces);

    }

    //Permet de tester si les pieces de la colonne numRow forment une combinaison gagnante
    public function isColWin(int $numCol) {

        $pieces = $this->plateau->getCol($numCol);

        return self::isComboWin($pieces);
    }

    //Permet de tester si les pieces de la zone  dir  forment une combinaison gagnante
    public function isCornerWin(int $dir) {
        $pieces = $this->plateau->getCorner($dir);

        return self::isComboWin($pieces);

    }

    //permet de vérifier si la pièce p peut être posée aux coordonnées rowNum et colNum lors du jeu
    public function isValidePose(int $rowNum, int $colNum,PieceQuantik $piece ):bool {
        $row = $this->plateau->getRow($rowNum); //on récupere les pieces de la ligne rowNum
        $col = $this->plateau->getCol($colNum); //on récupere les pieces de la colonne rowNum
        $dir = $this->plateau->getCornerFromCoord($rowNum,$colNum); //la zone ou se situe la piece de coordonnées rowNum et colNum
        $corner = $this->plateau->getCorner($dir); //on recupre les pieces de la zone dir

        if ( !$this->isPieceValide($row, $piece)) { //si la piece piece n'est pas jouable parmi les pieces contenues dans row on retourne false
            return false;
        }
        if ( !$this->isPieceValide($col, $piece)) { //si la piece piece n'est pas jouable parmi les pieces contenues dans col on retourne false
            return false;
        }
        if ( !$this->isPieceValide($corner, $piece)) { //si la piece piece n'est pas jouable parmi les pieces contenues dans corner on retourne false
            return false; 
        }
        return true; //si la piece piece n'est pas jouable parmi les pieces contenues dans rox,col, ou col on retourne true
    }

    //permet de deplacer une piece lors du jeu
    public function posePiece(int $rowNum, int $colNum,PieceQuantik $piece ) {
        $this->plateau->setPiece($rowNum, $colNum, $piece);
    }


    public function __toString() : string {
        return $this->plateau->__toString();//$this->getPlateau();
    }

    // Permet de tester si les pieces du tableau pieces  forment une combinaison gagnante
    private static function isComboWin(ArrayPieceQuantik $pieces) : bool{
        for($i =0;$i<$pieces->getTaille(); $i++ ) {
            if ($pieces->getPieceQuantik($i)->getForme() == PieceQuantik::initVoid()->getForme()) {
                return false;
            }
            for($j = $i+1; $j< $pieces->getTaille() ; $j++) {
                if($pieces->getPieceQuantik($i)->getForme() == $pieces->getPieceQuantik($j)->getForme() ) {
                    return false;
                }

            }
        }
        return true;

    }

    //permet de teste si la pièce p est jouable parmi les pieces du tableau piece (
    private static function isPieceValide(ArrayPieceQuantik $pieces, PieceQuantik $p) :bool {

        for($i=0; $i<$pieces->getTaille() ;$i++ ) {
            //vérifier si la piéce existe déjà
            if ($pieces->getPieceQuantik($i)->getForme() == $p->getForme()
                &&
                $pieces->getPieceQuantik($i)->getCouleur() != $p->getCouleur()
            ) {
                return false;
            }
        }

        return true;
    }

    //pour virifier s'il reste un case valide sur le plateau pour les pices contenu dans $pieces
    public function checkPosePossible( ArrayPieceQuantik $pieces) : bool{


        for ($i=0; $i < PlateauQuantik::NBROWS; $i++) {
            for ($j=0; $j < PlateauQuantik::NBROWS; $j++) {
                for ($k=0; $k <  $pieces->getTaille(); $k++) {
                    if ($this->isValidePose($i, $j, $pieces->getPieceQuantik($k))) {
                        return true;
                    }
                }
            }
        }

        return false;

    }
}

?>